﻿using Styx;
using Styx.Common;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.TreeSharp;
using Styx.Pathing;
using CommonBehaviors.Actions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;
using Styx.CommonBot;
using Styx.CommonBot.POI;

namespace TheLegionLooter
{
    public class TheLegionLooter : HBPlugin
    {
        public override string Author => "ElPargo";
        public override string Name => "The Legion Looter";
        public override Version Version => new Version(1, 1);
        public override string ButtonText => "TLL Settings";
        public override bool WantButton => true;
        public LocalPlayer Me => StyxWoW.Me;

        public static string UsedBot => BotManager.Current.Name.ToUpper();
        public static bool ValidBot => UsedBot.Contains("QUEST") || UsedBot.Contains("GATHER");
        private static readonly List<Crystal> Crystals = new List<Crystal>();
        public static string LastInfoLogMsg;
        private Composite _task;
        
        public override void OnButtonPress()
        {
            SettingsForm frm = new SettingsForm();
            frm.ShowDialog();
            TLLSet.Instance.Save();
        }

        internal async Task<bool> MainTask()
        {
            var scrap = GetFlowerScrap();
            if (scrap != null && !Me.Combat)
            {
                Navigator.MoveTo(scrap.Location);
                return true;
            }
            return false;
        }

        private static WoWObject GetChest()
        {
            return ObjectManager.GetObjectsOfTypeFast<WoWObject>()
                .Where(o => TLLSet.Instance.FindChests && o.Distance <= TLLSet.Instance.MaxRange && (IsEntryACrystal(o.Entry) || o.Name.Contains("Treasure Chest")))
                .OrderBy(o => o.Distance)
                .FirstOrDefault();
        }

        private static WoWUnit GetRares()
        {
            return ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                .Where(u => TLLSet.Instance.FindRares && u.Distance <= TLLSet.Instance.MaxRange && u.IsAlive && u.Classification == WoWUnitClassificationType.Rare)
                .OrderBy(u => u.Distance)
                .FirstOrDefault();
        }

        private static WoWAreaTrigger GetFlowerScrap()
        {
            IEnumerable<WoWAreaTrigger> t =
            from trigger in ObjectManager.GetObjectsOfType<WoWAreaTrigger>()
            where
                (int)trigger.Entry == 9756
            orderby
                trigger.DistanceSqr
            select
                trigger;

            return t.FirstOrDefault();
        }

        private static bool IsEntryACrystal(uint id)
        {
            foreach (var item in Crystals)
            {
                if (item.Id == id)
                { return true; }
            }
            return false;
        }

        public override void OnEnable()
        {
            base.OnEnable();
            Logging.Write("The Legion Looter: Enabled!");

            //Add Specific containers here.
            Crystals.Clear();
            Crystals.Add(new Crystal() { Id = 252774, Name = "Ancient Mana Crystal" });
            Crystals.Add(new Crystal() { Id = 245941, Name = "Warp Cache" });
            Crystals.Add(new Crystal() { Id = 252432, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252446, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252447, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252448, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252449, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 252450, Name = "Shimmering Ancient Mana Cluster" });
            Crystals.Add(new Crystal() { Id = 254006, Name = "Enchanted Burial Urn" });
            Crystals.Add(new Crystal() { Id = 254009, Name = "Volatile Leyline Crystal" });
            Crystals.Add(new Crystal() { Id = 254008, Name = "Kyrtos's Research Notes" });
            Crystals.Add(new Crystal() { Id = 248407, Name = "Kel'danath's Manaflask" });
            Crystals.Add(new Crystal() { Id = 254023, Name = "Arcane Power Unit" });
            Crystals.Add(new Crystal() { Id = 95958, Name = "Floating Treasure" });
            Crystals.Add(new Crystal() { Id = 245479, Name = "Battered Chest" });
            Crystals.Add(new Crystal() { Id = 240638, Name = "Disputed Treasure" });
            Crystals.Add(new Crystal() { Id = 241127, Name = "Unguarded Thistleleaf Treasure" });
            Crystals.Add(new Crystal() { Id = 241128, Name = "Unguarded Thistleleaf Treasure" });
            Crystals.Add(new Crystal() { Id = 253994, Name = "Seemingly Unguarded Treasure" });
            Crystals.Add(new Crystal() { Id = 252408, Name = "Ancient Mana Shard" });
            Crystals.Add(new Crystal() { Id = 252772, Name = "Ancient Mana Chunk" });

            _task = new ActionRunCoroutine(r => MainTask());
            TreeHooks.Instance.InsertHook("InGame_Check", 0, _task);
			
			InfoLog("By ElPargo | Version " + Version + " | [Enabled]");
        }

        public override void OnDisable()
        {
            base.OnDisable();
            TreeHooks.Instance.RemoveHook("InGame_Check", _task);
			InfoLog("[Disabled]");
        }

		public void InfoLog(string message, params object[] args)
        {
            if (message == LastInfoLogMsg) { return; }
            
            Logging.Write(Color.FromRgb(200, 255, 0), "[The Legion Looter]: {0}", message);
            LastInfoLogMsg = message;
        }

        public static void DebugLog(string message, params object[] args)
        { Logging.WriteQuiet(message, args); }

        public override void Pulse()
        {
            if (!StyxWoW.IsInGame || Me == null || !Me.IsValid || !Me.IsAlive || Me.OnTaxi || Me.InVehicle || Me.IsInInstance || Me.IsSwimming || Me.CurrentMap.ExpansionId != 6 || Me.ZoneId == 7502 || !ValidBot) { return; }

            var rare = GetRares();
            if (rare != null && !Me.IsCasting && !Blacklist.Contains(rare, BlacklistFlags.All))
            {
                InfoLog(rare.GetObjectName() + " Detected: Moving to Kill!");
                BotPoi.Current = new BotPoi(rare, PoiType.Kill, NavType.Fly);
            }

            var chest = GetChest();
            if (chest != null && rare == null && !Me.IsCasting && !Blacklist.Contains(chest, BlacklistFlags.All))
            {
                if ((chest.Entry == 252408 || chest.Entry == 252772) && !TLLSet.Instance.FindMana) { return; }

                InfoLog(chest.Guid + ": " + chest.GetObjectName() + " Detected: Moving to Loot!");
                BotPoi.Current = new BotPoi(chest, PoiType.Harvest, NavType.Fly);
            }
        }
    }

    public class Crystal
    {
        public uint Id { get; set; }
        public string Name { get; set; }
    }
}
